/*
 * $Id: rb_evas.c 354 2006-02-10 18:14:08Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Evas.h>

#define __RB_EVAS_C
#include "rb_evas_main.h"
#include "rb_evas.h"
#include "rb_evas_object.h"

VALUE cEvas;

static void c_mark (RbEvas *e)
{
	rb_gc_mark (e->parent);
}

static VALUE c_alloc (VALUE klass)
{
	RbEvas *evas = NULL;

	return Data_Make_Struct (cEvas, RbEvas, c_mark, free, evas);
}

VALUE TO_EVAS (VALUE parent, Evas *e)
{
	VALUE self;

	if (NIL_P (parent) || !e)
		return Qnil;

	self = rb_class_new_instance (0, NULL, cEvas);

	GET_OBJ (self, RbEvas, evas);

	evas->real = e;
	evas->parent = parent;

	return self;
}

/* :nodoc: */
static VALUE c_inspect (VALUE self)
{
	INSPECT (self, RbEvas);
}

/*
 * call-seq:
 *  e.render => nil
 *
 * Forces a re-render of the Evas.
 */
static VALUE c_render (VALUE self)
{
	GET_OBJ (self, RbEvas, e);

	evas_render (e->real);

	return Qnil;
}

static VALUE c_font_path_clear (VALUE self)
{
	GET_OBJ (self, RbEvas, e);

	evas_font_path_clear (e->real);

	return Qnil;
}

/*
 * call-seq:
 *  e.font_path_append(path) => nil
 *
 * Appends a path to the font path for <i>e</i>.
 */
static VALUE c_font_path_append (VALUE self, VALUE path)
{
	GET_OBJ (self, RbEvas, e);

	Check_Type (path, T_STRING);

	evas_font_path_append (e->real, StringValuePtr (path));

	return Qnil;
}

/*
 * call-seq:
 *  e.font_path_prepend(path) => nil
 *
 * Prepends a path to the font path for <i>e</i>.
 */
static VALUE c_font_path_prepend (VALUE self, VALUE path)
{
	GET_OBJ (self, RbEvas, e);

	Check_Type (path, T_STRING);

	evas_font_path_append (e->real, StringValuePtr (path));

	return Qnil;
}

/*
 * call-seq:
 *  e.font_path => array
 *
 * Returns the font path for <i>e</i>.
 */
static VALUE c_font_path_get (VALUE self)
{
	VALUE ary;
	const Evas_List *list, *l;

	GET_OBJ (self, RbEvas, e);

	if (!(list = evas_font_path_list (e->real)))
		return rb_ary_new ();

	ary = rb_ary_new2 (evas_list_count ((Evas_List *) list));

	for (l = list; l; l = l->next)
		rb_ary_push (ary, rb_str_new2 (l->data));

	return ary;
}
/*
 * call-seq:
 *  e.font_cache => fixnum
 *
 * Returns the size of the font cache for <i>e</i>.
 */
static VALUE c_font_cache_get (VALUE self)
 {
	GET_OBJ (self, RbEvas, e);

	return INT2FIX (evas_font_cache_get (e->real));
}

/*
 * call-seq:
 *  e.font_cache(fixnum)
 *
 * Sets the size of the font cache for <i>e</i>.
 */
static VALUE c_font_cache_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvas, e);

	Check_Type (val, T_FIXNUM);

	evas_font_cache_set (e->real, FIX2INT (val));

	return Qnil;
}

/*
 * call-seq:
 *  e.font_cache_reload => nil
 *
 * Flushes the font cache for <i>e</i>.
 */
static VALUE c_font_cache_flush (VALUE self)
{
	GET_OBJ (self, RbEvas, e);

	evas_font_cache_flush (e->real);

	return Qnil;
}

/*
 * call-seq:
 *  e.image_cache => fixnum
 *
 * Returns the size of the image cache for <i>e</i>.
 */
static VALUE c_image_cache_get (VALUE self)
{
	GET_OBJ (self, RbEvas, e);

	return INT2FIX (evas_image_cache_get (e->real));
}

/*
 * call-seq:
 *  e.image_cache(fixnum)
 *
 * Sets the size of the image cache for <i>e</i>.
 */
static VALUE c_image_cache_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvas, e);

	Check_Type (val, T_FIXNUM);

	evas_image_cache_set (e->real, FIX2INT (val));

	return Qnil;
}

/*
 * call-seq:
 *  e.image_cache_reload => nil
 *
 * Flushes the image cache for <i>e</i>.
 */
static VALUE c_image_cache_reload (VALUE self)
{
	GET_OBJ (self, RbEvas, e);

	evas_image_cache_reload (e->real);

	return Qnil;
}

/*
 * call-seq:
 *  e.image_cache_flush => nil
 *
 * Flushes the image cache for <i>e</i>.
 */
static VALUE c_image_cache_flush (VALUE self)
{
	GET_OBJ (self, RbEvas, e);

	evas_image_cache_flush (e->real);

	return Qnil;
}

/*
 * call-seq:
 *  e.top => evasobject
 *
 * Returns the <code>Evas::EvasObject</code> at the top of <i>e</i>.
 */
static VALUE c_top_get (VALUE self)
{
	Evas_Object *o;

	GET_OBJ (self, RbEvas, e);

	if (!(o = evas_object_top_get (e->real)))
		return Qnil;

	return TO_EVAS_OBJECT (o);
}

/*
 * call-seq:
 *  e.bottom => evasobject
 *
 * Returns the <code>Evas::EvasObject</code> at the bottom of <i>e</i>.
 */
static VALUE c_bottom_get (VALUE self)
{
	Evas_Object *o;

	GET_OBJ (self, RbEvas, e);

	if (!(o = evas_object_bottom_get (e->real)))
		return Qnil;

	return TO_EVAS_OBJECT (o);
}

/*
 * call-seq:
 *  e.find_object(name) => evasobject
 *
 * Returns the <code>Evas::EvasObject</code> with the name <i>name</i>.
 */
static VALUE c_find_object (VALUE self, VALUE name)
{
	Evas_Object *o;

	GET_OBJ (self, RbEvas, e);

	Check_Type (name, T_STRING);

	if (!(o = evas_object_name_find (e->real, StringValuePtr (name))))
		return Qnil;

	return TO_EVAS_OBJECT (o);
}

static VALUE c_output_size_get (VALUE self)
{
	int w = 0, h = 0;

	GET_OBJ (self, RbEvas, e);

	evas_output_size_get (e->real, &w, &h);

	return rb_ary_new3 (2, INT2FIX (w), INT2FIX (h));
}

static VALUE c_output_viewport_get (VALUE self)
{
	int x = 0, y = 0, w = 0, h = 0;

	GET_OBJ (self, RbEvas, e);

	evas_output_viewport_get (e->real,
	                          (Evas_Coord *) &x, (Evas_Coord *) &y,
	                          (Evas_Coord *) &w, (Evas_Coord *) &h);

	return rb_ary_new3 (4, INT2FIX (x), INT2FIX (y), INT2FIX (w),
	                    INT2FIX (h));
}

void Init_Evas (void)
{
	cEvas = rb_define_class_under (mEvas, "Evas", rb_cObject);

	rb_define_alloc_func (cEvas, c_alloc);

	/* not publically instantiable yet */
	rb_define_private_method (rb_singleton_class (cEvas),
	                          "new", NULL, 0);
	rb_define_method (cEvas, "inspect", c_inspect, 0);
	rb_define_method (cEvas, "render", c_render, 0);
	rb_define_method (cEvas, "font_path_clear", c_font_path_clear, 0);
	rb_define_method (cEvas, "font_path_append", c_font_path_append, 1);
	rb_define_method (cEvas, "font_path_prepend", c_font_path_prepend, 1);
	rb_define_method (cEvas, "font_path", c_font_path_get, 0);
	rb_define_method (cEvas, "font_cache", c_font_cache_get, 0);
	rb_define_method (cEvas, "font_cache=", c_font_cache_set, 1);
	rb_define_method (cEvas, "font_cache_flush",
	                  c_font_cache_flush, 0);
	rb_define_method (cEvas, "image_cache", c_image_cache_get, 0);
	rb_define_method (cEvas, "image_cache=", c_image_cache_set, 1);
	rb_define_method (cEvas, "image_cache_reload",
	                  c_image_cache_reload, 0);
	rb_define_method (cEvas, "image_cache_flush",
	                  c_image_cache_flush, 0);
	rb_define_method (cEvas, "top", c_top_get, 0);
	rb_define_method (cEvas, "bottom", c_bottom_get, 0);
	rb_define_method (cEvas, "find_object", c_find_object, 1);
	rb_define_method (cEvas, "output_size", c_output_size_get, 0);
	rb_define_method (cEvas, "output_viewport", c_output_viewport_get, 0);
}
